#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.mail`` -- Email Framework
"""

import smtplib
import warnings
import logging
from email.message import Message

from edbob.mail import sendmail_with_config


log = logging.getLogger(__name__)


def deprecation_warning(): # pragma no cover
    warnings.warn(
        "Configuration found in [edbob.mail] section, which is deprecated.  "
        "Please update configuration to use [rattail.mail] instead.",
        DeprecationWarning, stacklevel=2)


def send_message(config, sender, recipients, subject, body, content_type='text/plain'):
    """
    Assemble and deliver an email message using the given parameters and configuration.
    """
    message = make_message(sender, recipients, subject, body, content_type=content_type)
    deliver_message(config, message)


def make_message(sender, recipients, subject, body, content_type='text/plain'):
    """
    Assemble an email message object using the given parameters.
    """
    message = Message()
    message.set_type(content_type)
    message['From'] = sender
    for recipient in recipients:
        message['To'] = recipient
    message['Subject'] = subject
    message.set_payload(body)
    return message
    

def deliver_message(config, message):
    """
    Deliver an email message using the given SMTP configuration.
    """
    server = config.get('rattail.mail', 'smtp.server')
    if not server:
        server = config.get('edbob.mail', 'smtp.server')
        if server:
            deprecation_warning()
        else:
            server = 'localhost'

    username = config.get('rattail.mail', 'smtp.username')
    if not username:
        username = config.get('edbob.mail', 'smtp.username')
        if username:
            deprecation_warning()

    password = config.get('rattail.mail', 'smtp.password')
    if not password:
        password = config.get('edbob.mail', 'smtp.password')
        if password:
            deprecation_warning()

    log.debug("deliver_message: connecting to server: {0}".format(repr(server)))
    session = smtplib.SMTP(server)
    if username and password:
        result = session.login(username, password)
        log.debug("deliver_message: login result is: {0}".format(repr(result)))
    result = session.sendmail(message['From'], message.get_all('To'), message.as_string())
    log.debug("deliver_message: sendmail result is: {0}".format(repr(result)))
    session.quit()



#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
Core Stuff
"""

from uuid import uuid1


__all__ = ['Object', 'get_uuid']


class Object(object):
    """
    Generic base class.

    This is used mostly for convenience.
    """

    def __init__(self, **kwargs):
        """
        Constructor which assigns keyword arguments as attributes.
        """

        for key, value in kwargs.iteritems():
            setattr(self, key, value)

    def __getitem__(self, key):
        """
        Allows dict-like access to the object's attributes.
        """

        if hasattr(self, key):
            return getattr(self, key)

    def __str__(self):
        """
        Magic string method.

        This leverages :meth:`__unicode__()` if it exists; otherwise falls back
        to ``repr(self)``.
        """

        if hasattr(self, '__unicode__'):
            return str(unicode(self))
        return repr(self)


def get_uuid():
    """
    Generate a universally-unique identifier.

    :returns: A 32-character hex string.
    """

    return uuid1().hex



#!/usr/bin/env python

"""
``dtail.batches.products.labels`` -- Print Labels Batch
"""

from sqlalchemy.orm import object_session

import rattail
from ...db import model
from rattail.batches.providers import BatchProvider


class PrintLabels(BatchProvider):

    name = 'print_labels'
    description = "Print Labels"

    default_profile = None
    default_quantity = 1

    def add_columns(self, batch):
        batch.add_column('F01')
        batch.add_column('F155')
        batch.add_column('F02')
        batch.add_column('F22', display_name="Size")
        batch.add_column('F95', display_name="Label")
        batch.add_column('F94', display_name="Quantity")

    def add_rows_begin(self, batch, data):
        session = object_session(batch)
        if not self.default_profile:
            q = session.query(model.LabelProfile)
            q = q.order_by(model.LabelProfile.ordinal)
            self.default_profile = q.first()
            assert self.default_profile
        else:
            self.default_profile = session.merge(self.default_profile)

    def add_row(self, batch, product):
        row = batch.rowclass()
        row.F01 = product.upc
        if product.brand:
            row.F155 = product.brand.name
        row.F02 = product.description[:20]
        row.F22 = product.size
        row.F95 = self.default_profile.code
        row.F94 = self.default_quantity
        batch.add_row(row)

    def execute(self, batch, progress=None):
        prog = None
        if progress:
            prog = progress("Loading product data", batch.rowcount)

        session = object_session(batch)
        profiles = {}

        cancel = False
        for i, row in enumerate(batch.iter_rows(), 1):

            profile = profiles.get(row.F95)
            if not profile:
                q = session.query(model.LabelProfile)
                q = q.filter(model.LabelProfile.code == row.F95)
                profile = q.one()
                profile.labels = []
                profiles[row.F95] = profile

            q = session.query(model.Product)
            q = q.filter(model.Product.upc == row.F01)
            product = q.one()

            profile.labels.append((product, row.F94))

            if prog and not prog.update(i):
                cancel = True
                break

        if not cancel:
            for profile in profiles.itervalues():
                printer = profile.get_printer()
                if not printer.print_labels(profile.labels):
                    cancel = True
                    break

        if prog:
            prog.destroy()
        return not cancel

    def set_params(self, session, **params):
        profile = params.get('profile')
        if profile:
            q = session.query(model.LabelProfile)
            q = q.filter(model.LabelProfile.code == profile)
            self.default_profile = q.one()

        quantity = params.get('quantity')
        if quantity and quantity.isdigit():
            self.default_quantity = int(quantity)



#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
Database Stuff
"""

from edbob.db import Session

from .core import *
from .changes import *


def init(config):
    """
    Initialize the Rattail database framework.
    """

    import edbob
    import rattail

    # Pretend all ``edbob`` models come from Rattail, until that is true.
    from edbob.db import Base
    names = []
    for name in edbob.__all__:
        obj = getattr(edbob, name)
        if isinstance(obj, type) and issubclass(obj, Base):
            names.append(name)
    edbob.graft(rattail, edbob, names)

    # Pretend all ``edbob`` enumerations come from Rattail.
    from edbob import enum
    edbob.graft(rattail, enum)

    from rattail.db.extension import model
    edbob.graft(rattail, model)

    ignore_role_changes = config.getboolean(
        'rattail.db', 'changes.ignore_roles', default=True)

    if config.getboolean('rattail.db', 'changes.record'):
        record_changes(Session, ignore_role_changes)

    elif config.getboolean('rattail.db', 'record_changes'):
        import warnings
        warnings.warn("Config setting 'record_changes' in section [rattail.db] "
                      "is deprecated; please use 'changes.record' instead.",
                      DeprecationWarning)
        record_changes(Session, ignore_role_changes)




from unittest import TestCase

from mock import patch, Mock, MagicMock

from edbob.configuration import AppConfigParser

from rattail import mail


class TestMail(TestCase):

    @patch('rattail.mail.deliver_message')
    @patch('rattail.mail.make_message')
    def test_send_message(self, make_message, deliver_message):
        config = AppConfigParser('rattail')
        mail.send_message(config,
                          'sender@mailinator.com', ['recip1@mailinator.com', 'recip2@mailinator.com'],
                          'test subject', 'test body')
        make_message.assert_called_once_with(
            'sender@mailinator.com', ['recip1@mailinator.com', 'recip2@mailinator.com'],
            'test subject', 'test body', content_type='text/plain')
        deliver_message.assert_called_once_with(config, make_message.return_value)

    def test_make_message(self):
        message = mail.make_message('sender@mailinator.com', ['recip1@mailinator.com', 'recip2@mailinator.com'],
                                    'test subject', 'test body', content_type='text/html')
        self.assertEqual(message['From'], 'sender@mailinator.com')
        self.assertEqual(message.get_all('To'), ['recip1@mailinator.com', 'recip2@mailinator.com'])
        self.assertEqual(message['Subject'], 'test subject')
        self.assertEqual(message.get_content_type(), 'text/html')
        self.assertEqual(message.get_payload(), 'test body')


@patch('rattail.mail.smtplib')
class TestDeliverMessage(TestCase):
    
    def test_config_defaults(self, smtplib):
        config = AppConfigParser('rattail')
        message = MagicMock()
        session = Mock()
        smtplib.SMTP.return_value = session
        mail.deliver_message(config, message)
        smtplib.SMTP.assert_called_once_with('localhost')
        self.assertFalse(session.login.called)
        self.assertEqual(session.sendmail.call_count, 1)
        session.quit.assert_called_once_with()

    def test_config_custom(self, smtplib):
        config = AppConfigParser('rattail')
        config.set('rattail.mail', 'smtp.server', 'mail.mailinator.com')
        config.set('rattail.mail', 'smtp.username', 'foo')
        config.set('rattail.mail', 'smtp.password', 'bar')
        message = MagicMock()
        session = Mock()
        smtplib.SMTP.return_value = session
        mail.deliver_message(config, message)
        smtplib.SMTP.assert_called_once_with('mail.mailinator.com')
        session.login.assert_called_once_with('foo', 'bar')
        self.assertEqual(session.sendmail.call_count, 1)
        session.quit.assert_called_once_with()

    @patch('rattail.mail.deprecation_warning')
    def test_config_deprecated(self, deprecation_warning, smtplib):
        config = AppConfigParser('rattail')
        config.set('edbob.mail', 'smtp.server', 'mail.mailinator.com')
        config.set('edbob.mail', 'smtp.username', 'foo')
        config.set('edbob.mail', 'smtp.password', 'bar')
        message = MagicMock()
        session = Mock()
        smtplib.SMTP.return_value = session
        mail.deliver_message(config, message)
        smtplib.SMTP.assert_called_once_with('mail.mailinator.com')
        session.login.assert_called_once_with('foo', 'bar')
        self.assertEqual(session.sendmail.call_count, 1)
        session.quit.assert_called_once_with()
        self.assertEqual(deprecation_warning.call_count, 3)




from unittest import TestCase

from rattail import core


class TestCore(TestCase):

    def test_get_uuid(self):
        uuid = core.get_uuid()
        self.assertIsInstance(uuid, str)
        self.assertEqual(len(uuid), 32)
